/*
 * Copyright (c) 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.1.1 (build: 2011-10-24 19:10:53 PDT)
 */

package com.google.api.services.tasks;


import com.google.api.client.http.ByteArrayContent;
import com.google.api.client.http.HttpMethod;
import com.google.api.client.http.HttpRequest;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpResponse;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.util.GenericData;
import com.google.api.client.util.Key;
import java.io.IOException;


/**
 * Service definition for Tasks (v1).
 *
 * <p>
 * Lets you manage your tasks and task lists.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="http://code.google.com/apis/tasks/v1/using.html" target="_blank">API Documentation</a>
 * </p>
 *
 * @author Google, Inc.
 */
public class Tasks extends ApiClient {

  /**
   * Reference to myself so instances of inner classes can find the handle to their service
   * connection.
   */
  final Tasks self;

  /**
   * Construct a Tasks instance to connect to the Tasks service.
   *
   * @param transport A transport to use for subsequent API calls.
   * @param requestInitializer A initializer to use to build requests for subsequent API calls. May be null if no initializer is required.
   * @param jsonFactory A factory to deliver JSON parsers. The choice depends
   *   on the execution environment. Most applications will want to use the
   *   open-source Jackson library, via the factory
   *   com.google.api.client.json.jackson.JacksonFactory.
   *   Android applications will probably want to use the smaller GSON library,
   *   (com.google.api.client.json.gson.GsonFactory).
   */
  public Tasks(
      HttpTransport transport,
      HttpRequestInitializer requestInitializer,
      JsonFactory jsonFactory) {
    super("https://www.googleapis.com", "/tasks/v1/", transport, requestInitializer, jsonFactory);
    self = this;
  }

  /**
   * Construct a Tasks instance to connect to the Tasks service.
   *
   * @param transport A transport to use for subsequent API calls.
   * @param jsonFactory A factory to deliver JSON parsers. The choice depends
   *   on the execution environment. Most applications will want to use the
   *   open-source Jackson library, via the factory
   *   com.google.api.client.json.jackson.JacksonFactory.
   *   Android applications will probably want to use the smaller GSON library,
   *   (com.google.api.client.json.gson.GsonFactory).
   */
  public Tasks(HttpTransport transport, JsonFactory jsonFactory) {
    this(transport, null, jsonFactory);
  }

  /**
   * Construct a Tasks instance to connect to the Tasks service.
   *
   * @param appName The application name. This is embedded into the User-Agent string
   * @param transport A transport to use for subsequent API calls.
   * @param jsonFactory A factory to deliver JSON parsers. The choice depends
   *   on the execution environment. Most applications will want to use the
   *   open-source Jackson library, via the factory
   *   com.google.api.client.json.jackson.JacksonFactory.
   *   Android applications will probably want to use the smaller GSON library,
   *   (com.google.api.client.json.gson.GsonFactory).
   */
  public Tasks(String appName, HttpTransport transport, JsonFactory jsonFactory) {
    this(transport, null, jsonFactory);
    this.setApplicationName(appName);
  }


  /**
   * An accessor for creating requests from the TasksOperations collection.
   * The typical use is:<pre>
   *   {@code Tasks tasks = new Tasks(...);}
   *   {@code Tasks.TasksOperations.List request = tasks.tasks.list(parameters ...)}</pre>
   */
  public final TasksOperations tasks = new TasksOperations();

  /**
   * The "tasks" collection of methods.
   */
  public class TasksOperations {

    /**
     * Creates a new task on the specified task list.
     *
     * Create a request for the method "tasks.insert".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @param content the {@link Task}
     * @return the request
     */
    public Insert insert(String tasklist, com.google.api.services.tasks.model.Task content) {
      return new Insert(tasklist, content);
    }

    public class Insert extends RemoteRequest {

      private static final String REST_PATH = "lists/{tasklist}/tasks";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Insert(String tasklist, com.google.api.services.tasks.model.Task content) {
        this.tasklist = tasklist;
        this.content = content;
      }

      /**
       * Sends the "insert" request to the Tasks server.
       *
       * @return the {@link Task} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.Task execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.Task result = jsonParser.parse(
            response, com.google.api.services.tasks.model.Task.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "insert" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.POST,
                             REST_PATH,
                             content);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Insert setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }

      /**  Parent task identifier. If the task is created at the top level, this parameter is omitted.
      Optional.
       */
      @Key("parent")
      private String parent;

      /**
     * Parent task identifier. If the task is created at the top level, this parameter is omitted.
     * Optional.
     */
      public String getParent() {
        return parent;
      }

      /**
     * Parent task identifier. If the task is created at the top level, this parameter is omitted.
     * Optional.
     */
      public Insert setParent(String parent) {
        this.parent = parent;
        return this;
      }

      /**  Previous sibling task identifier. If the task is created at the first position among its siblings,
      this parameter is omitted. Optional.
       */
      @Key("previous")
      private String previous;

      /**
     * Previous sibling task identifier. If the task is created at the first position among its
     * siblings, this parameter is omitted. Optional.
     */
      public String getPrevious() {
        return previous;
      }

      /**
     * Previous sibling task identifier. If the task is created at the first position among its
     * siblings, this parameter is omitted. Optional.
     */
      public Insert setPrevious(String previous) {
        this.previous = previous;
        return this;
      }


      private com.google.api.services.tasks.model.Task content;

    }
    /**
     * Returns the specified task.
     *
     * Create a request for the method "tasks.get".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @param task Task identifier.
     * @return the request
     */
    public Get get(String tasklist, String task) {
      return new Get(tasklist, task);
    }

    public class Get extends RemoteRequest {

      private static final String REST_PATH = "lists/{tasklist}/tasks/{task}";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Get(String tasklist, String task) {
        this.tasklist = tasklist;
        this.task = task;
      }

      /**
       * Sends the "get" request to the Tasks server.
       *
       * @return the {@link Task} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.Task execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.Task result = jsonParser.parse(
            response, com.google.api.services.tasks.model.Task.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "get" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.GET,
                             REST_PATH);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Get setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }

      /**  Task identifier.
       */
      @Key("task")
      private String task;

      /** Task identifier. */
      public String getTask() {
        return task;
      }

      /** Task identifier. */
      public Get setTask(String task) {
        this.task = task;
        return this;
      }


    }
    /**
     * Clears all completed tasks from the specified task list. The affected tasks will be marked as
     * 'hidden' and no longer be returned by default when retrieving all tasks for a task list.
     *
     * Create a request for the method "tasks.clear".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Clear#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @return the request
     */
    public Clear clear(String tasklist) {
      return new Clear(tasklist);
    }

    public class Clear extends RemoteRequest {

      private static final String REST_PATH = "lists/{tasklist}/clear";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Clear(String tasklist) {
        this.tasklist = tasklist;
      }

      /**
       * Sends the "clear" request to the Tasks server.
       *
       * @throws IOException if the request fails
       */
      public void execute() throws IOException {
        HttpResponse response = executeUnparsed();

        response.ignore();

      }

      /**
       * Sends the "clear" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.POST,
                             REST_PATH);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Clear setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }


    }
    /**
     * Moves the specified task to another position in the task list. This can include putting it as a
     * child task under a new parent and/or move it to a different position among its sibling tasks.
     *
     * Create a request for the method "tasks.move".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Move#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @param task Task identifier.
     * @return the request
     */
    public Move move(String tasklist, String task) {
      return new Move(tasklist, task);
    }

    public class Move extends RemoteRequest {

      private static final String REST_PATH = "lists/{tasklist}/tasks/{task}/move";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Move(String tasklist, String task) {
        this.tasklist = tasklist;
        this.task = task;
      }

      /**
       * Sends the "move" request to the Tasks server.
       *
       * @return the {@link Task} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.Task execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.Task result = jsonParser.parse(
            response, com.google.api.services.tasks.model.Task.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "move" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.POST,
                             REST_PATH);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Move setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }

      /**  Task identifier.
       */
      @Key("task")
      private String task;

      /** Task identifier. */
      public String getTask() {
        return task;
      }

      /** Task identifier. */
      public Move setTask(String task) {
        this.task = task;
        return this;
      }

      /**  New parent task identifier. If the task is moved to the top level, this parameter is omitted.
      Optional.
       */
      @Key("parent")
      private String parent;

      /**
     * New parent task identifier. If the task is moved to the top level, this parameter is omitted.
     * Optional.
     */
      public String getParent() {
        return parent;
      }

      /**
     * New parent task identifier. If the task is moved to the top level, this parameter is omitted.
     * Optional.
     */
      public Move setParent(String parent) {
        this.parent = parent;
        return this;
      }

      /**  New previous sibling task identifier. If the task is moved to the first position among its
      siblings, this parameter is omitted. Optional.
       */
      @Key("previous")
      private String previous;

      /**
     * New previous sibling task identifier. If the task is moved to the first position among its
     * siblings, this parameter is omitted. Optional.
     */
      public String getPrevious() {
        return previous;
      }

      /**
     * New previous sibling task identifier. If the task is moved to the first position among its
     * siblings, this parameter is omitted. Optional.
     */
      public Move setPrevious(String previous) {
        this.previous = previous;
        return this;
      }


    }
    /**
     * Returns all tasks in the specified task list.
     *
     * Create a request for the method "tasks.list".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @return the request
     */
    public List list(String tasklist) {
      return new List(tasklist);
    }

    public class List extends RemoteRequest {

      private static final String REST_PATH = "lists/{tasklist}/tasks";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private List(String tasklist) {
        this.tasklist = tasklist;
      }

      /**
       * Sends the "list" request to the Tasks server.
       *
       * @return the {@link Tasks} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.Tasks execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.Tasks result = jsonParser.parse(
            response, com.google.api.services.tasks.model.Tasks.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "list" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.GET,
                             REST_PATH);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public List setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }

      /**  Upper bound for a task's due date (as a RFC 3339 timestamp) to filter by. Optional. The default is
      not to filter by due date.
       */
      @Key("dueMax")
      private String dueMax;

      /**
     * Upper bound for a task's due date (as a RFC 3339 timestamp) to filter by. Optional. The
     * default is not to filter by due date.
     */
      public String getDueMax() {
        return dueMax;
      }

      /**
     * Upper bound for a task's due date (as a RFC 3339 timestamp) to filter by. Optional. The
     * default is not to filter by due date.
     */
      public List setDueMax(String dueMax) {
        this.dueMax = dueMax;
        return this;
      }

      /**  Flag indicating whether deleted tasks are returned in the result. Optional. The default is False.
       */
      @Key("showDeleted")
      private Boolean showDeleted;

      /**
     * Flag indicating whether deleted tasks are returned in the result. Optional. The default is
     * False.
     */
      public Boolean getShowDeleted() {
        return showDeleted;
      }

      /**
     * Flag indicating whether deleted tasks are returned in the result. Optional. The default is
     * False.
     */
      public List setShowDeleted(Boolean showDeleted) {
        this.showDeleted = showDeleted;
        return this;
      }

      /**  Lower bound for a task's last modification time (as a RFC 3339 timestamp) to filter by. Optional.
      The default is not to filter by last modification time.
       */
      @Key("updatedMin")
      private String updatedMin;

      /**
     * Lower bound for a task's last modification time (as a RFC 3339 timestamp) to filter by.
     * Optional. The default is not to filter by last modification time.
     */
      public String getUpdatedMin() {
        return updatedMin;
      }

      /**
     * Lower bound for a task's last modification time (as a RFC 3339 timestamp) to filter by.
     * Optional. The default is not to filter by last modification time.
     */
      public List setUpdatedMin(String updatedMin) {
        this.updatedMin = updatedMin;
        return this;
      }

      /**  Lower bound for a task's completion date (as a RFC 3339 timestamp) to filter by. Optional. The
      default is not to filter by completion date.
       */
      @Key("completedMin")
      private String completedMin;

      /**
     * Lower bound for a task's completion date (as a RFC 3339 timestamp) to filter by. Optional.
     * The default is not to filter by completion date.
     */
      public String getCompletedMin() {
        return completedMin;
      }

      /**
     * Lower bound for a task's completion date (as a RFC 3339 timestamp) to filter by. Optional.
     * The default is not to filter by completion date.
     */
      public List setCompletedMin(String completedMin) {
        this.completedMin = completedMin;
        return this;
      }

      /**  Maximum number of task lists returned on one page. Optional. The default is 100.
       */
      @Key("maxResults")
      private Long maxResults;

      /** Maximum number of task lists returned on one page. Optional. The default is 100. */
      public Long getMaxResults() {
        return maxResults;
      }

      /** Maximum number of task lists returned on one page. Optional. The default is 100. */
      public List setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /**  Flag indicating whether completed tasks are returned in the result. Optional. The default is True.
       */
      @Key("showCompleted")
      private Boolean showCompleted;

      /**
     * Flag indicating whether completed tasks are returned in the result. Optional. The default is
     * True.
     */
      public Boolean getShowCompleted() {
        return showCompleted;
      }

      /**
     * Flag indicating whether completed tasks are returned in the result. Optional. The default is
     * True.
     */
      public List setShowCompleted(Boolean showCompleted) {
        this.showCompleted = showCompleted;
        return this;
      }

      /**  Token specifying the result page to return. Optional.
       */
      @Key("pageToken")
      private String pageToken;

      /** Token specifying the result page to return. Optional. */
      public String getPageToken() {
        return pageToken;
      }

      /** Token specifying the result page to return. Optional. */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**  Upper bound for a task's completion date (as a RFC 3339 timestamp) to filter by. Optional. The
      default is not to filter by completion date.
       */
      @Key("completedMax")
      private String completedMax;

      /**
     * Upper bound for a task's completion date (as a RFC 3339 timestamp) to filter by. Optional.
     * The default is not to filter by completion date.
     */
      public String getCompletedMax() {
        return completedMax;
      }

      /**
     * Upper bound for a task's completion date (as a RFC 3339 timestamp) to filter by. Optional.
     * The default is not to filter by completion date.
     */
      public List setCompletedMax(String completedMax) {
        this.completedMax = completedMax;
        return this;
      }

      /**  Flag indicating whether hidden tasks are returned in the result. Optional. The default is False.
       */
      @Key("showHidden")
      private Boolean showHidden;

      /**
     * Flag indicating whether hidden tasks are returned in the result. Optional. The default is
     * False.
     */
      public Boolean getShowHidden() {
        return showHidden;
      }

      /**
     * Flag indicating whether hidden tasks are returned in the result. Optional. The default is
     * False.
     */
      public List setShowHidden(Boolean showHidden) {
        this.showHidden = showHidden;
        return this;
      }

      /**  Lower bound for a task's due date (as a RFC 3339 timestamp) to filter by. Optional. The default is
      not to filter by due date.
       */
      @Key("dueMin")
      private String dueMin;

      /**
     * Lower bound for a task's due date (as a RFC 3339 timestamp) to filter by. Optional. The
     * default is not to filter by due date.
     */
      public String getDueMin() {
        return dueMin;
      }

      /**
     * Lower bound for a task's due date (as a RFC 3339 timestamp) to filter by. Optional. The
     * default is not to filter by due date.
     */
      public List setDueMin(String dueMin) {
        this.dueMin = dueMin;
        return this;
      }


    }
    /**
     * Updates the specified task.
     *
     * Create a request for the method "tasks.update".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @param task Task identifier.
     * @param content the {@link Task}
     * @return the request
     */
    public Update update(String tasklist, String task, com.google.api.services.tasks.model.Task content) {
      return new Update(tasklist, task, content);
    }

    public class Update extends RemoteRequest {

      private static final String REST_PATH = "lists/{tasklist}/tasks/{task}";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Update(String tasklist, String task, com.google.api.services.tasks.model.Task content) {
        this.tasklist = tasklist;
        this.task = task;
        this.content = content;
      }

      /**
       * Sends the "update" request to the Tasks server.
       *
       * @return the {@link Task} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.Task execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.Task result = jsonParser.parse(
            response, com.google.api.services.tasks.model.Task.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "update" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.PUT,
                             REST_PATH,
                             content);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Update setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }

      /**  Task identifier.
       */
      @Key("task")
      private String task;

      /** Task identifier. */
      public String getTask() {
        return task;
      }

      /** Task identifier. */
      public Update setTask(String task) {
        this.task = task;
        return this;
      }


      private com.google.api.services.tasks.model.Task content;

    }
    /**
     * Updates the specified task. This method supports patch semantics.
     *
     * Create a request for the method "tasks.patch".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @param task Task identifier.
     * @param content the {@link Task}
     * @return the request
     */
    public Patch patch(String tasklist, String task, com.google.api.services.tasks.model.Task content) {
      return new Patch(tasklist, task, content);
    }

    public class Patch extends RemoteRequest {

      private static final String REST_PATH = "lists/{tasklist}/tasks/{task}";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Patch(String tasklist, String task, com.google.api.services.tasks.model.Task content) {
        this.tasklist = tasklist;
        this.task = task;
        this.content = content;
      }

      /**
       * Sends the "patch" request to the Tasks server.
       *
       * @return the {@link Task} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.Task execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.Task result = jsonParser.parse(
            response, com.google.api.services.tasks.model.Task.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "patch" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.PATCH,
                             REST_PATH,
                             content);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Patch setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }

      /**  Task identifier.
       */
      @Key("task")
      private String task;

      /** Task identifier. */
      public String getTask() {
        return task;
      }

      /** Task identifier. */
      public Patch setTask(String task) {
        this.task = task;
        return this;
      }


      private com.google.api.services.tasks.model.Task content;

    }
    /**
     * Deletes the specified task from the task list.
     *
     * Create a request for the method "tasks.delete".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @param task Task identifier.
     * @return the request
     */
    public Delete delete(String tasklist, String task) {
      return new Delete(tasklist, task);
    }

    public class Delete extends RemoteRequest {

      private static final String REST_PATH = "lists/{tasklist}/tasks/{task}";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Delete(String tasklist, String task) {
        this.tasklist = tasklist;
        this.task = task;
      }

      /**
       * Sends the "delete" request to the Tasks server.
       *
       * @throws IOException if the request fails
       */
      public void execute() throws IOException {
        HttpResponse response = executeUnparsed();

        response.ignore();

      }

      /**
       * Sends the "delete" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.DELETE,
                             REST_PATH);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Delete setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }

      /**  Task identifier.
       */
      @Key("task")
      private String task;

      /** Task identifier. */
      public String getTask() {
        return task;
      }

      /** Task identifier. */
      public Delete setTask(String task) {
        this.task = task;
        return this;
      }


    }

  }

  /**
   * An accessor for creating requests from the Tasklists collection.
   * The typical use is:<pre>
   *   {@code Tasks tasks = new Tasks(...);}
   *   {@code Tasks.Tasklists.List request = tasks.tasklists.list(parameters ...)}</pre>
   */
  public final Tasklists tasklists = new Tasklists();

  /**
   * The "tasklists" collection of methods.
   */
  public class Tasklists {

    /**
     * Creates a new task list and adds it to the authenticated user's task lists.
     *
     * Create a request for the method "tasklists.insert".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link TaskList}
     * @return the request
     */
    public Insert insert(com.google.api.services.tasks.model.TaskList content) {
      return new Insert(content);
    }

    public class Insert extends RemoteRequest {

      private static final String REST_PATH = "users/@me/lists";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Insert(com.google.api.services.tasks.model.TaskList content) {
        this.content = content;
      }

      /**
       * Sends the "insert" request to the Tasks server.
       *
       * @return the {@link TaskList} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.TaskList execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.TaskList result = jsonParser.parse(
            response, com.google.api.services.tasks.model.TaskList.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "insert" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.POST,
                             REST_PATH,
                             content);
      }


      private com.google.api.services.tasks.model.TaskList content;

    }
    /**
     * Returns the authenticated user's specified task list.
     *
     * Create a request for the method "tasklists.get".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @return the request
     */
    public Get get(String tasklist) {
      return new Get(tasklist);
    }

    public class Get extends RemoteRequest {

      private static final String REST_PATH = "users/@me/lists/{tasklist}";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Get(String tasklist) {
        this.tasklist = tasklist;
      }

      /**
       * Sends the "get" request to the Tasks server.
       *
       * @return the {@link TaskList} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.TaskList execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.TaskList result = jsonParser.parse(
            response, com.google.api.services.tasks.model.TaskList.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "get" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.GET,
                             REST_PATH);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Get setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }


    }
    /**
     * Returns all the authenticated user's task lists.
     *
     * Create a request for the method "tasklists.list".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() {
      return new List();
    }

    public class List extends RemoteRequest {

      private static final String REST_PATH = "users/@me/lists";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private List() {
      }

      /**
       * Sends the "list" request to the Tasks server.
       *
       * @return the {@link TaskLists} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.TaskLists execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.TaskLists result = jsonParser.parse(
            response, com.google.api.services.tasks.model.TaskLists.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "list" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.GET,
                             REST_PATH);
      }

      /**  Token specifying the result page to return. Optional.
       */
      @Key("pageToken")
      private String pageToken;

      /** Token specifying the result page to return. Optional. */
      public String getPageToken() {
        return pageToken;
      }

      /** Token specifying the result page to return. Optional. */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**  Maximum number of task lists returned on one page. Optional. The default is 100.
       */
      @Key("maxResults")
      private Long maxResults;

      /** Maximum number of task lists returned on one page. Optional. The default is 100. */
      public Long getMaxResults() {
        return maxResults;
      }

      /** Maximum number of task lists returned on one page. Optional. The default is 100. */
      public List setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }


    }
    /**
     * Updates the authenticated user's specified task list.
     *
     * Create a request for the method "tasklists.update".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @param content the {@link TaskList}
     * @return the request
     */
    public Update update(String tasklist, com.google.api.services.tasks.model.TaskList content) {
      return new Update(tasklist, content);
    }

    public class Update extends RemoteRequest {

      private static final String REST_PATH = "users/@me/lists/{tasklist}";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Update(String tasklist, com.google.api.services.tasks.model.TaskList content) {
        this.tasklist = tasklist;
        this.content = content;
      }

      /**
       * Sends the "update" request to the Tasks server.
       *
       * @return the {@link TaskList} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.TaskList execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.TaskList result = jsonParser.parse(
            response, com.google.api.services.tasks.model.TaskList.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "update" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.PUT,
                             REST_PATH,
                             content);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Update setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }


      private com.google.api.services.tasks.model.TaskList content;

    }
    /**
     * Updates the authenticated user's specified task list. This method supports patch semantics.
     *
     * Create a request for the method "tasklists.patch".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @param content the {@link TaskList}
     * @return the request
     */
    public Patch patch(String tasklist, com.google.api.services.tasks.model.TaskList content) {
      return new Patch(tasklist, content);
    }

    public class Patch extends RemoteRequest {

      private static final String REST_PATH = "users/@me/lists/{tasklist}";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Patch(String tasklist, com.google.api.services.tasks.model.TaskList content) {
        this.tasklist = tasklist;
        this.content = content;
      }

      /**
       * Sends the "patch" request to the Tasks server.
       *
       * @return the {@link TaskList} response
       * @throws IOException if the request fails
       */
      public com.google.api.services.tasks.model.TaskList execute() throws IOException {
        HttpResponse response = executeUnparsed();

        com.google.api.services.tasks.model.TaskList result = jsonParser.parse(
            response, com.google.api.services.tasks.model.TaskList.class);
        result.setResponseHeaders(response.getHeaders());
        return result;

      }

      /**
       * Sends the "patch" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.PATCH,
                             REST_PATH,
                             content);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Patch setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }


      private com.google.api.services.tasks.model.TaskList content;

    }
    /**
     * Deletes the authenticated user's specified task list.
     *
     * Create a request for the method "tasklists.delete".
     *
     * This request holds the parameters needed by the the tasks server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param tasklist Task list identifier.
     * @return the request
     */
    public Delete delete(String tasklist) {
      return new Delete(tasklist);
    }

    public class Delete extends RemoteRequest {

      private static final String REST_PATH = "users/@me/lists/{tasklist}";

      /**
       * Internal constructor.  Use the convenience method instead.
       */
      private Delete(String tasklist) {
        this.tasklist = tasklist;
      }

      /**
       * Sends the "delete" request to the Tasks server.
       *
       * @throws IOException if the request fails
       */
      public void execute() throws IOException {
        HttpResponse response = executeUnparsed();

        response.ignore();

      }

      /**
       * Sends the "delete" request to the Tasks server and returns the raw {@link HttpResponse}.
       * @return the {@link HttpResponse}
       * @throws IOException if the request fails
       */
      public HttpResponse executeUnparsed() throws IOException {
        return super.execute(HttpMethod.DELETE,
                             REST_PATH);
      }

      /**  Task list identifier.
       */
      @Key("tasklist")
      private String tasklist;

      /** Task list identifier. */
      public String getTasklist() {
        return tasklist;
      }

      /** Task list identifier. */
      public Delete setTasklist(String tasklist) {
        this.tasklist = tasklist;
        return this;
      }


    }

  }



  /**
   * Base class for all service requests.
   *
   * <p>This class is the base for all requests defined by the tasks service. It provides methods to
   * build URLs for request objects, execute the the remote call and invoke the parser to return
   * results.</p>
   */
  class RemoteRequest extends GenericData {
    @Key
    private String fields;

    RemoteRequest() {
      fields = Tasks.this.getFields();
    }

    /**
     * Returns the partial field mask, or <code>null</code> if none is set.
     */
    public String getFields() {
      return fields;
    }

    /**
     * Sets the partial field mask for this request and response.
     */
    public void setFields(String fields) {
      this.fields = fields;
    }

    HttpResponse execute(HttpMethod method, String requestUrl) throws IOException {
      HttpRequest request = self.buildHttpRequest(method, requestUrl, this);
      if (method == HttpMethod.POST || method == HttpMethod.PUT || method == HttpMethod.PATCH) {
        request.setContent(ByteArrayContent.fromString("application/json", " "));
      }
      return request.execute();
    }

    HttpResponse execute(HttpMethod method, String requestUrl, Object body) throws IOException {
      HttpRequest request = self.buildHttpRequest(method, requestUrl, this);
      request.setContent(self.createSerializer(body));
      request.setEnableGZipContent(true);
      return request.execute();
    }
  }
}
